from django.http.response import HttpResponse, HttpResponseForbidden
from django.shortcuts import render

# Create your views here.
from django.contrib.auth.decorators import login_required
import requests
import urllib.parse
import json
from django.conf import settings 
from rest_framework.decorators import api_view, permission_classes
from rest_framework import generics
from .permissions import IsAuthenticated
from .remote_model import RemoteModel

@api_view(['GET'])
def get_user(request):
    if request.user.is_authenticated:
        user = RemoteModel(request,'authentication','api/auth/user/list').get()
        return HttpResponse(user)
    else:
        return HttpResponse('not login')

class GetUser(generics.GenericAPIView):
    permission_classes = [IsAuthenticated]
    def get(self,request,*args,**kwargs):
        user = RemoteModel(request,'auth','api/auth/user/list').get()
        return HttpResponse(user)



from rest_framework import permissions,status
from django.conf import settings

from .remote_model import RemoteModel
import jwt
# to give access to different user
import json
from rest_framework.exceptions import APIException

class TokenValidation:
    def verify_token(request):
        try:
            token = request.headers['Authorization']
        except KeyError as e:
            return False
        if request.session.has_key("user") and request.session["user"] == token:
            return True
        verify = RemoteModel(request, 'auth', 'verify_token', token).verify_token(token)
        perm = (verify.text)
        if verify.status_code == 200:
            request.session["user"] = token
            request.session["user_type"] = perm
            return True
        else:
            raise InvalidToken()


    def get_user_permission(request,user_type,perm):
        try:
            token = request.headers['Authorization']
            if request.session.has_key("user") and request.session["user"] == token:
                try:
                    verify = RemoteModel(request, 'user_perm', 'check_perm', token).get_permission(user_type,perm)
                    if verify.status_code == 200:
                        return True
                    else:
                        return False
                except Exception as e:
                    print(e)
                    return False
            else:
                return False
        except KeyError as e:
            return False        


class IsAuthenticated(permissions.BasePermission):
    """
    Allows access only to authenticated users. It takes
    the token from headers and send it to auth service to
    verify. If verified it creates session for that user inorder
    to reduce the number of requests for verifying token. If not
    in session then create new session.If no token provided return False.
    """ 

    def has_permission(self, request, view):
        token = TokenValidation.verify_token(request)
        return True if token else False
            


class SetFacebook(permissions.BasePermission):

    def has_permission(self, request, view):
        """
        checking whether normal user has permission to view or not
        """
       
        user = request.session['user_type'].strip('"')
        permission = 'facebook_settings'
        user_perm = bool(TokenValidation.get_user_permission(request,user,permission))
        if user_perm:
            return True
        else:   
            # return False
            raise Forbidden()
        


class InvalidToken(APIException):
    """
        responds 400 for invalid or expired tokens
    """
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = 'Token is invalid or expired'
    default_code = 'token_not_valid'


class Forbidden(APIException):
    """
        responds 400 for invalid or expired tokens
    """
    status_code = status.HTTP_401_UNAUTHORIZED
    default_detail = 'user has no permission to access this data'
    default_code = 'permission_not_allowed'



import requests
import json
from django.conf import settings 

class RemoteModel:
    def __init__(self, request, entity, endpoint,token = None,tenant = None):
        self.request = request
        self.entity = entity
        self.endpoint = endpoint
        self.override_headers = {'Authorization':token}
      
        self.tenant = str(str(request.tenant)+'.') if request.tenant else ''
        self.url = f'{settings.PROTOCOL}{self.tenant}{settings.ENTITY_BASE_URL_MAP.get(entity)}/{settings.ENTITY_URL_PATH.get(endpoint)}'

    def _headers(self, override_headers=None):
        base_headers = {'content-type': 'application/json'}
        override_headers = self.override_headers or {}
        return {
            # **self,
            **base_headers,
            **override_headers,
        }
        
    def _cookies(self, override_cookies=None):
        override_cookies = override_cookies or {}
        return {
        **self.request.COOKIES,
        **override_cookies,
        }

    def verify_token(self,token):
        url = f'{self.url}/'
        try:
            data = {'token':token.split(' ')[1]}
        except:
            data = {'token':token}
        return requests.post(
        url,json.dumps(data),
        headers=self._headers(),
        cookies=self._cookies())
    
    def get_permission(self,ut,perm = None):
        url = f'{self.url}/{ut}/'
        data = {'permission':perm}
        return requests.get(
        url,data,
        headers=self._headers(),
        cookies=self._cookies())

    def get_detail(self, entity_id):
        url = f'{self.url}/{entity_id}'
        return requests.get(
        url,
        headers=self._headers(),
        cookies=self._cookies())
    
    def get(self):
        url = f'{self.url}/'
        return requests.get(
        url,
        headers=self._headers(),
        cookies=self._cookies())

    def delete(self, entity_id):
        url = f'{self.url}/{entity_id}/'

        return requests.delete(
        url,
        headers=self._headers(),
        cookies=self._cookies())

    def update(self, entity_id, entity_data):
        url = f'{self.url}/{entity_id}/'
        return requests.put(
        url,
        data=json.dumps(entity_data),
        headers=self._headers(),
        cookies=self._cookies())

    def create(self, entity_data):
        url = f'{self.url}/'
        return requests.post(url,
        data=json.dumps(entity_data),
        headers=self._headers(),
        cookies=self._cookies())

